#!/usr/bin/perl

###  check_fusionreactor.pl

# A Nagios plugin using the Nagios::Plugin modules.  

# Framework by Nathan Vonnahme
# n8v at users dot sourceforge dot net, July 19 2006
#
# Extended for FusionReactor
# sales at fusion-reactor dot com, March 10 2009

##############################################################################
# prologue
use strict;
use warnings;

use Nagios::Plugin ;

use vars qw($VERSION $PROGNAME  $verbose $warn $critical $timeout $result);
$VERSION = '1.0';

# get the base name of this script for use in the examples
use File::Basename;
$PROGNAME = basename($0);


##############################################################################
# define and get the command line options.
#   see the command line option guidelines at 
#   http://nagiosplug.sourceforge.net/developer-guidelines.html#PLUGOPTIONS


# Instantiate Nagios::Plugin object (the 'usage' parameter is mandatory)
my $p = Nagios::Plugin->new(
    usage => "Usage: %s
    [-v|--verbose]
    [-H <host>]
    [--port <port>]
    [--path <path>]
    [-l <login name>]
    [--password <password>]
    [--field <xml field>]
    [-c|--critical=<critical threshold>] 
    [-w|--warning=<warning threshold>]",  
    version => $VERSION,
    blurb => 'This plugin checks a FusionReactor value.', 

	extra => "

THRESHOLDs for -w and -c are specified 'min:max' or 'min:' or ':max'
(or 'max'). If specified '\@min:max', a warning status will be generated
if the count *is* inside the specified range.

See more threshold examples at http
  : // nagiosplug
  . sourceforge
  . net / developer-guidelines
  . html    #THRESHOLDFORMAT
  "
);


# Define and document the valid command line options
# usage, help, version, timeout and verbose are defined by default.

$p->add_arg(
	spec => 'warning|w=s',

	help => 
qq{-w, --warning=INTEGER:INTEGER
   Minimum and maximum number of allowable result, outside of which a
   warning will be generated.  If omitted, no warning is generated.},

	required => 1,
#	default => 10,
);

$p->add_arg(
	spec => 'critical|c=s',
	help => 
qq{-c, --critical=INTEGER:INTEGER
   Minimum and maximum number of the generated result, outside of
   which a critical will be generated. },

	required => 1
);

$p->add_arg(
        spec => 'host|H=s',
        help =>
qq{-H, --host=STRING
   The FR host name. },
	required => 1
);

$p->add_arg(
        spec => 'port=i',
        help =>
qq{--port=INTEGER
   The FR port number. },
	required => 1
);

$p->add_arg(
        spec => 'path=s',
        help =>
qq{--path=STRING
   The FR path name. },
	required => 1
);

$p->add_arg(
        spec => 'l=s',
        help =>
qq{--l=STRING
   The FR user login. },
	required => 1
);

$p->add_arg(
        spec => 'password=s',
        help =>
qq{--password=STRING
   The FR password. },
	required => 1
);

$p->add_arg(
        spec => 'field=s',
        help =>
qq{--field=STRING
   The FR XML field. },
	required => 1
);


# Parse arguments and process standard ones (e.g. usage, help, version)
$p->getopts;


# perform sanity checking on command line options
if ( !(defined $p->opts->host) )  {
    $p->nagios_die( " no host supplied " );
}
if ( !(defined $p->opts->warning) )  {
    $p->nagios_die( " no warning level supplied " );
}
if ( !(defined $p->opts->critical) )  {
    $p->nagios_die( " no critical level supplied " );
}
if ( !(defined $p->opts->port) )  {
    $p->nagios_die( " no port supplied " );
}
if ( !(defined $p->opts->path) )  {
    $p->nagios_die( " no path supplied " );
}
if ( !(defined $p->opts->l) )  {
    $p->nagios_die( " no login supplied " );
}
if ( !(defined $p->opts->password) )  {
    $p->nagios_die( " no password supplied " );
}
if ( !(defined $p->opts->field) )  {
    $p->nagios_die( " no field supplied " );
}

unless ( defined $p->opts->warning || defined $p->opts->critical ) {
	$p->nagios_die( " you didn't supply a threshold argument " );
}



##############################################################################
# check stuff.

# THIS is where you'd do your actual checking to get a real value for $result
#  don't forget to timeout after $p->opts->timeout seconds, if applicable.
#my $result;
#if (defined $p->opts->result) {  # you got a 'result' option from the command line options
#    $result = $p->opts->result;
#    print " using supplied result $result from command line \n
#  " if $p->opts->verbose;
#}
#else {
#    $result = int rand(20)+1;
#    print " generated random result $result\n " if $p->opts->verbose;
#}

    use LWP::UserAgent;
    use URI::URL;

my $uri;
$uri = 'http://' . $p->opts->host . ':' . $p->opts->port . $p->opts->path . 'fService.cfm?command=remoting&subcommand=transfer&data=metrics';

sub subChar {
	my($message) = @_;
	my $i;
	my $code;
	my $char;
	my $ret = '';

	for ($i=0; $i<length $message; $i++)
	{
		$code = substr($message, $i, 1);
		$char = ord($code);
		if ($char >= ord('a') && $char <= ord('m')) {
			$char += 13;
		} elsif ($char >= ord('n') && $char <= ord('z')) {
                        $char -= 13;
                } elsif ($char >= ord('A') && $char <= ord('M')) {
                        $char += 13;
                } elsif ($char >= ord('N') && $char <= ord('Z')) {
                        $char -= 13;
                } elsif ($char >= ord('0') && $char <= ord('4')) {
                        $char += 5;
                } elsif ($char >= ord('5') && $char <= ord('9')) {
                        $char -= 5;
		}

		$ret = $ret . chr($char);
	}
	
	return $ret;
}
sub trim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

use Digest::MD5;
my $ctx = Digest::MD5->new;
$ctx->add($p->opts->password);
my $md5Hash = $ctx->hexdigest;

my $upperMD5 = uc( $md5Hash );

my $substituted = subChar( $upperMD5 );

my $fullString = 'NoUser:' . $substituted;

use MIME::Base64;
my $authString = encode_base64( $fullString );
$authString = trim($authString);

my $hdrs = new HTTP::Headers(Accept => 'text/plain',
                             'User-Agent' => 'NagiosBrowser/1.0',
			     'Authorization' => 'Basic ' . $authString);



my $ua   = new LWP::UserAgent;
my $url  = new URI::URL($uri);
my $req  = new HTTP::Request('GET', $uri, $hdrs);
my $resp = $ua->request($req);
my $code = $resp->content;

my $result = $code;

    use XML::XPath;
    use XML::XPath::XMLParser;
    
    my $xp = XML::XPath->new(xml => $code);

$result = $xp->findvalue( $p->opts->field )->value();


##############################################################################
# check the result against the defined warning and critical thresholds,
# output the result and exit
$p->nagios_exit( 
	 return_code => $p->check_threshold($result), 
	 message => "Result was $result" 
);

